//
// Created by fabio on 9/13/24.
//

#ifndef DRIP_H
#define DRIP_H

#include <opendroneid.h>
#include "utils.h"

#define DRIP_MAX_SAM_LEN 200
#define DRIP_SERIAL_NUMBER_LEN 21
#define DRIP_HHIT_LEN 16
#define DRIP_MAX_MANIFEST_MSGS 11
#define DRIP_HASH_LEN 8
#define DRIP_VNX_TIMESTAMP_LEN sizeof(int32_t)
#define DRIP_UA_SIGNATURE_LEN 64
#define DRIP_ED25519_KEY_LEN 32
#define DRIP_WRAPPER_MAX_MSG 4
#define DRIP_FRAME_EVIDENCE_DATA_LENGTH 111
#define DRIP_ENDORSEMENT_LEN 136

#ifdef __cplusplus
extern "C" {
#endif

// HostIdentity structure definition
typedef struct {
    unsigned char hhit[DRIP_HHIT_LEN];
    unsigned char priv_key[DRIP_ED25519_KEY_LEN];
    unsigned char pub_key[DRIP_ED25519_KEY_LEN];
    unsigned char serial_number[DRIP_SERIAL_NUMBER_LEN];
    int is_hhit_verified;
} DRIP_HostIdentity;

typedef struct {
    uint8_t legacy_transport;
} DRIP_Core_Config;

typedef struct {
    uint8_t frame_type;
    uint8_t frame_evidence_data[DRIP_FRAME_EVIDENCE_DATA_LENGTH];
} DRIP_Frame_data;

// Providing only this generic structure useful for decoding because in most case evidence length is variable
// thus it is not possible to provide a fixed structure
enum SAM_TYPE {
    DRIP_LINK = 0x01,
    DRIP_WRAPPER = 0x02,
    DRIP_MANIFEST = 0x03,
    DRIP_FRAME = 0x04,
};

typedef struct {
    unsigned char sam_type;
    unsigned char sam_auth_data[DRIP_MAX_SAM_LEN];
} DRIP_SAM_Data_Format;





//Host Identity API
DRIP_HostIdentity* DRIP_HostIdentityFromBytes(const unsigned char* priv_key, const unsigned char* pub_key, const unsigned char* hhit);

/** Internal API function will need to sign secure data. This function needs to be called before using the API
 *  First param: DripHostIdentity containing public key
 *  Second param: Input data that needs to be signed
 *  Third param: Length in bytes of input data
 *  Fourth param: Output data buffer
 *  Fifth param: Output data len
 */
void DRIP_SetSignDataFun(void (*fun)(DRIP_HostIdentity*, const unsigned char*, const size_t, unsigned char*, size_t*));


/** Internal API function will need current date time. This function needs to be called before using the API
 *  Pass in a pointer to a function returning the current time time_t.
 */
void DRIP_SetASTMDateTimeFun(time_t (*fun)());

/*  DRIP Core API might in future need more internal configuration this function is be used to set the configuration
 *  Currently it only sets whether the legacy transport is enabled or not
 */
void DRIP_SetConfig(DRIP_Core_Config* config);






// DRIP Encoding API
//todo docs
int encodeDripLink(
    ODID_Auth_encoded *encoded,
    const char *endorsement,
    short *pages_out
);

int encodeDripManifest(
    ODID_Auth_encoded *encoded,
    DRIP_HostIdentity *drone_hi,
    ODID_Message_encoded *messages,
    size_t cnt,
    const unsigned char *previous_manifest_hash,
    const unsigned char *drip_link_be_hda_ua_hash,
    unsigned char *next_manifest_hash_out,
    short *pages_out
);

int encodeDripWrapper(
    ODID_Auth_encoded *encoded,
    DRIP_HostIdentity *drone_hi,
    ODID_Message_encoded *messages,
    size_t cnt,
    short *pages_out
);

int encodeDripFrame(
    ODID_Auth_encoded *encoded,
    DRIP_HostIdentity *drone_hi,
    const DRIP_Frame_data *data,
    size_t evidence_len,
    short *pages_out
);

#ifdef __cplusplus
}
#endif

#endif //DRIP_H
