//
// Created by fabio on 9/15/24.
//

#include "utils.h"

#include "drip.h"
#include <sp800-185.h>
#include <stdio.h>


void cshake128_wrapper(uint8_t *output,
    const uint8_t *input,
    size_t inlen) {

    static const unsigned char context_id[16] = {0x00, 0xB5, 0xA6, 0x9C, 0x79, 0x5D, 0xF5, 0xD5, 0xF0, 0x08, 0x7F, 0x56, 0x84, 0x3F, 0x2C, 0x40};
    // Compute hash of the combined data
    cshake128(output, DRIP_HASH_LEN, NULL, 0, context_id, 16, input, inlen);
}

int hex_string_to_bytes(const char *hex_str, unsigned char *bytes, size_t *bytes_len) {
    size_t len = strlen(hex_str);

    // Hex string length must be even
    if (len % 2 != 0) {
        fprintf(stderr, "Invalid hex string length.\n");
        return -1;
    }

    // Calculate the number of bytes
    *bytes_len = len / 2;

    for (size_t i = 0; i < *bytes_len; i++) {
        sscanf(hex_str + 2*i, "%2hhx", &bytes[i]);
    }

    return 0;
}

// Function to convert a byte array to an integer (big-endian)
unsigned long long int from_bytes(const unsigned char* bytes, size_t len) {
    unsigned long long int result = 0;
    for (size_t i = 0; i < len; i++) {
        result = (result << 8) | bytes[i];
    }
    return result;
}

// Function to convert an integer to a binary string and pad to 75 bits
char* to_binary_string_padded(unsigned long long int value, int bit_length) {
    char* binary_string = malloc(bit_length + 1); // +1 for null terminator
    if (!binary_string) {
        return NULL;
    }

    for (int i = 0; i < bit_length; i++) {
        binary_string[bit_length - 1 - i] = (value & (1ULL << i)) ? '1' : '0';
    }
    binary_string[bit_length] = '\0';  // Null terminator

    return binary_string;
}

int compareODIDMessages(const void * p, const void * arg) {
    ODID_Message_encoded *a = (ODID_Message_encoded*)p;
    ODID_Message_encoded *b = (ODID_Message_encoded*)arg;

    return a->rawData[1] - b->rawData[1];
}