//
// Created by fabio on 9/26/24.
//


#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <drip.h>
#include <assert.h>

#include "test_utils.h"

static char identities_path[512];
static DRIP_HostIdentity *drone = NULL;
static DRIP_HostIdentity *hda = NULL;

static void parse_command_line(int argc, char *argv[]) {
    if (argc == 1) {
        printf("--identities-path arg not specified\n");
        exit(EXIT_SUCCESS);
    }

    if(!strcmp(argv[1], "--identities-path")) {
        int p_len = strlen(argv[2]);
        if(p_len<512)
            strncpy(identities_path, argv[2], p_len);
        else {
            printf("--identities-path arg is too long\n");
            exit(EXIT_FAILURE);
        }
    }
    else {
        printf("--identities-path arg not specified\n");
        exit(EXIT_FAILURE);
    }
}

/*
 * Tests are quite weak at the moment, and improvements are welcomed.
 */

int test_fec() {
    printf("Initializing drip-core...\n");
    DRIP_Core_Config config;

    //enable legacy transport for FEC testing
    config.legacy_transport = 1;
    DRIP_SetConfig(&config);

    DRIP_SetSignDataFun(&sign_data);

    //test forward error correction with BasicID drip frame
    printf("Testing FEC...\n");

    //Create message
    ODID_BasicID_data basic_id_1 = {0};
    basic_id_1.IDType = ODID_IDTYPE_SPECIFIC_SESSION_ID;
    basic_id_1.UAType = ODID_UATYPE_HELICOPTER_OR_MULTIROTOR;
    memcpy(basic_id_1.UASID, drone->hhit, DRIP_HHIT_LEN);

    //encode it with opendroneid api
    ODID_Message_encoded encoded;
    if(encodeBasicIDMessage((ODID_BasicID_encoded *)&encoded, &basic_id_1) != ODID_SUCCESS) {
        perror("Failed to encode BasicID 1");
        return 0;
    }

    //encode it with drip api using wrapper (just for quick testing, could be done with manifest)
    ODID_Auth_encoded encoded_wrapper[16];
    short wrapper_pages;
    if(encodeDripWrapper(encoded_wrapper, drone, &encoded, 1, &wrapper_pages) != ODID_SUCCESS) {
        perror("Failed to encode Drip Wrapper");
        return 0;
    }


    //decode with opendroneidapi
    ODID_Auth_data decoded[wrapper_pages];

    for(int i = 0; i<wrapper_pages; i++) {
        if(decodeAuthMessage(&decoded[i], &encoded_wrapper[i]) != ODID_SUCCESS) {
            perror("Failed to decode Drip Wrapper");
            return 0;
        }
    }

    uint8_t decoded_len = decoded[0].Length - ODID_AUTH_PAGE_ZERO_DATA_SIZE;

    uint8_t additional_pages = decoded_len % ODID_AUTH_PAGE_NONZERO_DATA_SIZE == 0 ? 2 : 1;
    uint8_t additional_data_len = 0;
    if(additional_pages == 1) {
        additional_data_len = decoded[decoded[0].LastPageIndex-1].AuthData[decoded_len % 23];
    }
    else
        additional_data_len = decoded[decoded[0].LastPageIndex-1].AuthData[0];

    assert(additional_data_len == ODID_AUTH_PAGE_NONZERO_DATA_SIZE);



    //calculate FEC and check that is equal to the one in last page
    uint8_t fec[ODID_AUTH_PAGE_NONZERO_DATA_SIZE] = {0};
    for(int i = 0; i<wrapper_pages-additional_pages; i++) {
        for(int j = 0; j<ODID_AUTH_PAGE_NONZERO_DATA_SIZE; j++) {
            //todo might not be up to standard +2 skips the header
            fec[j] ^= ((uint8_t*)&encoded_wrapper[i])[j+2];
        }
    }
    assert(!memcmp(fec, decoded[decoded[0].LastPageIndex].AuthData, ODID_AUTH_PAGE_NONZERO_DATA_SIZE));



    //reconstruct one page of the encoded message
    ODID_Auth_encoded reconstructed = {0};
    //todo The header is placeholded given that FEC is not used for the header
    reconstructed.page_non_zero.MessageType = ODID_MESSAGETYPE_AUTH;
    reconstructed.page_non_zero.ProtoVersion = 2;
    reconstructed.page_non_zero.AuthType = ODID_AUTH_SPECIFIC_AUTHENTICATION;
    reconstructed.page_non_zero.DataPage = 2;


    //use fec to reconstruct the third page with rolling xor
    for(int i = 0; i<wrapper_pages-additional_pages; i++) {
        for(int j = 2; j<ODID_AUTH_PAGE_NONZERO_DATA_SIZE+2; j++) {
            if(i != 2) //skip the page to be reconstructed
                //todo might not be up to standard +2 skips the header
                ((uint8_t*)&reconstructed)[j] ^= ((uint8_t*)&encoded_wrapper[i])[j];
        }
    }

    //add the fec to the reconstructed page
    for(int j = 0; j<ODID_AUTH_PAGE_NONZERO_DATA_SIZE; j++)
        ((uint8_t*)&reconstructed)[j+2] ^= decoded[decoded[0].LastPageIndex].AuthData[j];

    //assert that the decoded message is the same as the original after reconstruction
    //todo might not be up to standard +2 skips the header
    assert(!memcmp(&reconstructed, &encoded_wrapper[2], ODID_AUTH_PAGE_NONZERO_DATA_SIZE+2));


    uint8_t* reconstructed_ptr = (uint8_t*)&reconstructed;
    uint8_t* encoded_wrapper_ptr = (uint8_t*)&encoded_wrapper[2];

    printf("Reconstructed memory dump:\n");
    //todo might not be up to standard +2 skips the header
    for (int k = 1; k < ODID_AUTH_PAGE_NONZERO_DATA_SIZE+2; k++) {
        printf("%02X ", reconstructed_ptr[k]);
    }
    printf("\n");

    printf("Encoded wrapper memory dump:\n");
    //todo might not be up to standard +2 skips the header
    for (int k = 1; k < ODID_AUTH_PAGE_NONZERO_DATA_SIZE+2; k++) {
        printf("%02X ", encoded_wrapper_ptr[k]);
    }
    printf("\n");


    printf("FEC test passed\n");
    return 0;
}

int main(int argc, char **argv)
{
    parse_command_line(argc, argv);
    drone = DRIP_HostIdentityFromFile(identities_path,"drone");
    hda = DRIP_HostIdentityFromFile(identities_path,"hda");

    printf("Running tests...\n");
    //future test call could be added here and &&'ed to the return value
    //just a suggestion, better organization is welcomed
    int ret = test_fec();

    return ret;
}